interface Message {
  text: string;
  location: Location | null;
  notes: Note[];

  // Optional user-specified data that is passed through unmodified. You can
  // use this to stash the original error, for example.
  detail: any;
}

interface Note {
  text: string;
  location: Location | null;
}

interface Location {
  file: string;
  namespace: string;
  line: number; // 1-based
  column: number; // 0-based, in bytes
  length: number; // in bytes
  lineText: string;
}

interface OutputFile {
  path: string;
  contents: Uint8Array; // "text" as bytes
  text: string; // "contents" as text
}

interface BuildInvalidate {
  (): Promise<BuildIncremental>;
  dispose(): void;
}

interface BuildIncremental extends BuildResult {
  rebuild: BuildInvalidate;
}

interface BuildResult {
  warnings: Message[];
  outputFiles?: OutputFile[]; // Only when "write: false"
  rebuild?: BuildInvalidate; // Only when "incremental: true"
  stop?: () => void; // Only when "watch: true"
}

/** Mark some properties as required, leaving others unchanged */
declare type MarkRequired<T, RK extends keyof T> = Exclude<T, RK> & Required<Pick<T, RK>>;

declare type Format = 'cjs' | 'esm' | 'iife';
declare type Options = {
    entryPoints?: string[];
    /**
     * Output different formats to differen folder instead of using different extensions
     */
    legacyOutput?: boolean;
    /**
     * Compile target, like `es2018`
     */
    target?: string;
    minify?: boolean;
    minifyWhitespace?: boolean;
    minifyIdentifiers?: boolean;
    minifySyntax?: boolean;
    keepNames?: boolean;
    watch?: boolean;
    jsxFactory?: string;
    jsxFragment?: string;
    outDir?: string;
    format?: Format[];
    globalName?: string;
    env?: {
        [k: string]: string;
    };
    define?: {
        [k: string]: string;
    };
    dts?: boolean | string | {
        entry?: string;
        /** Resolve external types used in dts files from node_modules */
        resolve?: boolean | (string | RegExp)[];
    };
    sourcemap?: boolean;
    /** Don't bundle these packages */
    external?: string[];
    /** Transform the result with `@babel/core` */
    babel?: boolean;
    /**
     * Replace `process.env.NODE_ENV` with `production` or `development`
     * `production` when the bundled is minified, `development` otherwise
     */
    replaceNodeEnv?: boolean;
};
declare type NormalizedOptions = MarkRequired<Options, 'entryPoints' | 'format'>;
declare const makeLabel: (input: string, type: 'info' | 'success' | 'error') => string;
declare function runEsbuild(options: Options, { format }: {
    format: Format;
}): Promise<BuildResult | undefined>;
declare function build(_options: Options): Promise<void>;

export { Format, NormalizedOptions, Options, build, makeLabel, runEsbuild };
